import { css } from '@emotion/css';
import { useEffect } from 'react';
import { FormProvider, useFieldArray, useForm } from 'react-hook-form';

import { dateTime, GrafanaTheme2 } from '@grafana/data';
import { Button, Divider, Input, Stack, useStyles2, Field } from '@grafana/ui';
import { t } from 'app/core/internationalization';
import {
  ReportBaseV2,
  ReportIntervalFrequency,
  ReportSchedulingFrequencyV2,
  SendTime,
} from 'app/extensions/types/reports';

import { DashboardScene } from '../../../features/dashboard-scene/scene/DashboardScene';
import { DEFAULT_EMAIL_MESSAGE, defaultZoom } from '../constants';
import { selectors } from '../e2e-selectors/selectors';
import { getTimezone } from '../state/reducers';
import { canEditReport } from '../utils/permissions';
import { formSchemaValidationRules } from '../utils/validation';

import { SendPreviewToggletip } from './components/SendPreviewToggletip';
import Attachments from './sections/Attachments';
import EmailConfiguration from './sections/EmailConfiguration';
import Recipients from './sections/Recipients';
import { Schedule } from './sections/Schedule/Schedule';
import { intervalOptions } from './sections/Schedule/SendLaterSchedule';
import SelectDashboards from './sections/SelectDashboards/SelectDashboards';
import { useSelectDashboards } from './sections/SelectDashboards/useSelectDashboards';

const REPORT_FORM_SECTIONS = [
  { id: 'select-dashboards', component: SelectDashboards },
  { id: 'schedule', component: Schedule },
  { id: 'email-configuration', component: EmailConfiguration },
  { id: 'recipients', component: Recipients },
  { id: 'attachments', component: Attachments },
];

const defaultReport: Partial<ReportBaseV2> = {
  message: DEFAULT_EMAIL_MESSAGE,
  addDashboardUrl: true,
  addDashboardImage: false,
  pdfOptions: {
    orientation: 'landscape',
    layout: 'grid',
    scaleFactor: defaultZoom,
  },
  schedule: {
    sendTime: SendTime.Later,
    frequency: ReportSchedulingFrequencyV2.Daily,
    startDate: new Date(),
    endDate: '',
    endTime: dateTime(),
    startTime: dateTime().add(1, 'hour').startOf('hour'),
    timeZone: getTimezone(),
  },
};

export default function ReportForm({
  report,
  dashboard,
}: {
  report?: Partial<ReportBaseV2>;
  dashboard?: DashboardScene;
}) {
  const styles = useStyles2(getStyles);

  const form = useForm<ReportBaseV2>({
    mode: 'onChange',
    defaultValues: {
      ...defaultReport,
      ...report,
      dashboards: [],
      schedule: {
        ...defaultReport.schedule,
        ...report?.schedule,
        intervalFrequency:
          intervalOptions.find((opt) => opt.value === report?.schedule?.intervalFrequency)?.value ||
          ReportIntervalFrequency.Hours,
      },
    },
    shouldUnregister: true,
    shouldFocusError: true,
  });

  const { append: addDashboard, remove: removeDashboard } = useFieldArray({
    control: form.control,
    name: 'dashboards',
  });

  const { sceneDashboards, onAddDashboardClick } = useSelectDashboards({
    dashboardScene: dashboard,
    addDashboard,
    removeDashboard,
    dashboards: form.watch('dashboards'),
  });

  useEffect(() => {
    form.setFocus('title');
  }, [form, form.setFocus]);

  const onSubmit = (data: ReportBaseV2) => {
    // const dashboardsDTO = getDashboardsDTO(sceneDashboards);
    // console.log({
    //   ...data,
    //   dashboards: dashboardsDTO,
    //   schedule: {
    //     ...data.schedule,
    //     ...(data.schedule.startTime && {
    //       startTime: { hour: data.schedule.startTime?.hour?.(), minute: data.schedule.startTime?.minute?.() },
    //     }),
    //     ...(data.schedule.endTime && {
    //       endTime: { hour: data.schedule.endTime?.hour?.(), minute: data.schedule.endTime?.minute?.() },
    //     }),
    //   },
    // });
  };

  return (
    <FormProvider {...form}>
      <form
        onSubmit={form.handleSubmit(onSubmit)}
        className={styles.form}
        data-testid={selectors.components.ReportFormDrawer.container}
      >
        <fieldset disabled={!canEditReport} className={styles.container}>
          <Field
            label={t('share-report.report-name.label', 'Report name')}
            required
            error={form.formState.errors.title?.message}
            invalid={!!form.formState.errors.title}
          >
            <Input
              {...form.register('title', formSchemaValidationRules().title)}
              placeholder={report?.title}
              id="report-name"
              type="text"
              autoComplete="off"
              name={'name'}
            />
          </Field>
          <Divider spacing={0} />
          <div className={styles.reportSectionContainer}>
            <Stack direction="column" flex={1}>
              {REPORT_FORM_SECTIONS.map((section, index) => (
                <div key={section.id}>
                  <section.component dashboards={sceneDashboards} onAddDashboard={onAddDashboardClick} />
                  {index < REPORT_FORM_SECTIONS.length - 1 && <Divider />}
                </div>
              ))}
            </Stack>
          </div>
          <Divider />
          <ReportActions />
        </fieldset>
      </form>
    </FormProvider>
  );
}

function ReportActions() {
  return (
    <Stack justifyContent="space-between">
      <Stack gap={1}>
        <Button variant="secondary" fill="outline" icon="ellipsis-v" />
        <SendPreviewToggletip />
      </Stack>
      <Stack gap={1}>
        <Button type="submit" variant="primary" data-testid={selectors.components.ReportFormDrawer.submitButton}>
          {t('share-report.schedule-report.button', 'Schedule report')}
        </Button>
        <Button variant="secondary">{t('share-report.save-draft.button', 'Save draft')}</Button>
      </Stack>
    </Stack>
  );
}

const getStyles = (theme: GrafanaTheme2) => {
  return {
    form: css({
      height: '100%',
    }),
    container: css({
      display: 'flex',
      flexDirection: 'column',
      height: '100%',
    }),
    reportSectionContainer: css({
      height: '100%',
      marginTop: theme.spacing(2),
    }),
  };
};
